import logging
from urllib.parse import urljoin

import requests

from si_tests import logger

LOG = logger.logger

logger = logging.getLogger(__name__)


class HttpClient(object):
    def __init__(self, base_url=None, user=None, password=None):
        self.base_url = base_url
        self.kwargs = {}
        if user and password:
            self.kwargs.update({"auth": (user, password)})

    def set_base_url(self, base_url):
        self.base_url = base_url

    def request(self, url, method, headers=None, body=None,
                raise_on_error=True, **kwargs):
        logger.debug(
            "Sending request to: {}, body: {}, headers: {}, kwargs: {}".format(
                url, body, headers, kwargs))

        kwargs.update(self.kwargs)
        r = requests.request(method, urljoin(self.base_url, url),
                             headers=headers, data=body, **kwargs)
        if raise_on_error and not r.ok and r.status_code != 401:
            logger.error("Response code: {}, content: {}, "
                         "headers: {}".format(r.status_code,
                                              r.content,
                                              r.headers))
            raise requests.HTTPError(r.content)
        return r

    def post(self, url, headers=None, body=None, **kwargs):
        return self.request(url, "POST", headers=headers, body=body, **kwargs)

    def get(self, url, **kwargs):
        return self.request(url, "GET", **kwargs)

    def put(self, url, body=None, **kwargs):
        return self.request(url, "PUT", body=body, **kwargs)

    def delete(self, url, **kwargs):
        return self.request(url, "DELETE", **kwargs)
