#    Copyright 2020 Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations

from __future__ import absolute_import

from si_tests.clients.k8s.base import K8sBaseResource
from si_tests.clients.k8s.base import K8sBaseManager

from . import models


class Issuer(K8sBaseResource):
    resource_type = 'issuer'

    def _read(self, **kwargs):
        response = self._manager.api.get_cluster_custom_object(
            group=self._manager.resource_group,
            version=self._manager.resource_version,
            name=self.name,
            plural=self._manager.resource_plural,
            **kwargs)
        return self._manager._deserialize_custom_object(
            response, models.V1Issuer)

    def _create(self, body, **kwargs):
        response = self._manager.api.create_cluster_custom_object(
            group=self._manager.resource_group,
            version=self._manager.resource_version,
            plural=self._manager.resource_plural,
            body=body,
            **kwargs)
        return self._manager._deserialize_custom_object(
            response, models.V1Issuer)

    def _patch(self, body, **kwargs):
        response = self._manager.api.patch_cluster_custom_object(
            group=self._manager.resource_group,
            version=self._manager.resource_version,
            plural=self._manager.resource_plural,
            name=self.name,
            body=body,
            **kwargs)
        return self._manager._deserialize_custom_object(
            response, models.V1Issuer)

    def _replace(self, body, **kwargs):
        response = self._manager.api.replace_cluster_custom_object(
            group=self._manager.resource_group,
            version=self._manager.resource_version,
            plural=self._manager.resource_plural,
            name=self.name,
            body=body,
            **kwargs)
        return self._manager._deserialize_custom_object(
            response, models.V1Issuer)

    def _delete(self, body=None, **kwargs):
        if not body:
            body = {}
        self._manager.api.delete_cluster_custom_object(
            group=self._manager.resource_group,
            version=self._manager.resource_version,
            plural=self._manager.resource_plural,
            name=self.name,
            body=body,
            **kwargs)

    @property
    def data(self):
        """Returns dict of k8s object

        Data contains keys like api_version, kind, metadata,
        spec, status or items
        """
        return self.read().to_dict()


class IssuerManager(K8sBaseManager):
    resource_class = Issuer
    resource_group = 'cert-manager.io'
    resource_version = 'v1'
    resource_plural = 'issuers'

    @property
    def api(self):
        return self._cluster.api_custom

    def _list(self, namespace, **kwargs):
        response = self.api.list_namespaced_custom_object(
            group=self.resource_group,
            version=self.resource_version,
            namespace=namespace,
            plural=self.resource_plural,
            **kwargs)
        return self._deserialize_custom_object_list(
            response,
            models.V1IssuerList,
            models.V1Issuer)

    def _list_all(self, **kwargs):
        response = self.api.list_cluster_custom_object(
            group=self.resource_group,
            version=self.resource_version,
            plural=self.resource_plural,
            **kwargs)
        return self._deserialize_custom_object_list(
            response,
            models.V1IssuerList,
            models.V1Issuer)
