import exec_helpers
import pytest
import yaml
import json

from si_tests import settings
from si_tests import logger
import si_tests.utils.templates as utils
from si_tests.utils.utils import Provider

LOG = logger.logger


def test_kaas_ui(kaas_manager, kaas_ui_prerequisites):
    pod_name = settings.KAAS_UI_TEST_POD_NAME
    pod_namespace = settings.KAAS_UI_TEST_NAMESPACE
    lb_host = None
    writer_password = kaas_manager.get_keycloak_user_password('writer')

    mgmt_cluster = kaas_manager.get_mgmt_cluster()
    if mgmt_cluster.provider == Provider.baremetal:
        child_data = yaml.safe_load(
            utils.render_template(settings.BM_CHILD_SETTINGS_YAML_PATH))[0]
        # FIXME(alexz): This is wrong hardcode that should be fixed in 2.15+. see story PRODX-11290 PRODX-20493
        lb_host = child_data['child_config'].get('loadbalancer_host', None)
        cloud_provider = 'bm'
    elif mgmt_cluster.provider == Provider.openstack:
        cloud_provider = 'openstack'
    elif mgmt_cluster.provider == Provider.aws:
        cloud_provider = 'aws'
    elif mgmt_cluster.provider == Provider.vsphere:
        cloud_provider = 'vsphere'
    elif mgmt_cluster.provider == Provider.equinixmetal:
        cloud_provider = 'equinix'
    elif mgmt_cluster.provider == Provider.equinixmetalv2:
        cloud_provider = 'equinixmetalv2'
    elif mgmt_cluster.provider == Provider.azure:
        cloud_provider = 'azure'
    else:
        raise Exception("Unknown provider: {}".format(mgmt_cluster.provider))

    pod_list = kaas_manager.api.api_core.list_namespaced_pod(
        namespace='kaas',
        label_selector='app.kubernetes.io/instance=kaas-ui').to_dict()[
        'items'][0]

    kaas_version = pod_list['spec']['containers'][0]['image'].split(':')[1]

    # Temporarily pin 1.6.0 kaas-ui version
    # PRODX-4690
    if kaas_version == '1.5.0':
        kaas_version = '1.6.0'

    # Delete the pod before test if it is running
    if kaas_manager.api.pods.present(name=pod_name, namespace=pod_namespace):
        LOG.info("Pod '{0}' already exists, deleting".format(pod_name))
        pod = kaas_manager.api.pods.get(name=pod_name, namespace=pod_namespace)
        pod.delete()

    kaas_service = kaas_manager.api.api_core.read_namespaced_service(
        name='kaas-kaas-ui',
        namespace='kaas').to_dict()
    LOG.debug(kaas_service)
    kaas_ui_ip = \
        kaas_service['status']['load_balancer']['ingress'][0]['ip'] or \
        kaas_service['status']['load_balancer']['ingress'][0]['hostname']
    cdn_region = settings.CDN_REGION
    artifacts_region_map = settings.ARTIFACTS_REGION_MAPPING
    ui_image_base = artifacts_region_map[cdn_region]["images_base_url"]
    options = {
        'KAAS_UI_TEST_IMAGE': ui_image_base + '/' +
        settings.KAAS_UI_TEST_IMAGE + ':' +
        kaas_version,
        'LOADBALANCER_HOST': lb_host,
        'KAAS_UI_ADDRESS': kaas_ui_ip,
        'KAAS_UI_USERNAME': 'writer',
        'KAAS_UI_PASSWORD': writer_password,
        'CLOUD_PROVIDER': cloud_provider,
        'KAAS_UI_PORT': kaas_service['spec']['ports'][0]['port'],
        'TEST_TYPE': 'e2e'
    }

    templates = utils.render_template(settings.KAAS_UI_TEST_POD_YAML, options)
    LOG.debug(templates)
    json_body = json.dumps(yaml.load(templates, Loader=yaml.SafeLoader))

    pod = kaas_manager.api.pods.create(name=pod_name,
                                       namespace=pod_namespace,
                                       body=json.loads(json_body))
    pod.wait_phase('Running')

    try:
        pod.wait_test(filepath='./test_*', timeout=5400)
    finally:
        exec_helpers.Subprocess().check_call(
            "mkdir -p {}/reports".format(settings.ARTIFACTS_DIR))

        res = pod.exec(
            ['/bin/sh', '-c', 'ls -la {0}/reports'.format(
                 settings.KAAS_UI_TEST_REPORTS_DIR)])
        LOG.info("Reports in {0}/reports :\n{1}"
                 .format(settings.KAAS_UI_TEST_REPORTS_DIR, res))

        pod.cp_from_pod(
            source_dir=settings.KAAS_UI_TEST_REPORTS_DIR + '/reports',
            destination=settings.ARTIFACTS_DIR + '/reports'
        )
        pod.cp_from_pod(
            source_dir=settings.KAAS_UI_TEST_REPORTS_DIR + '/screenshots'
        )
        pod.cp_from_pod(
            source_dir=settings.KAAS_UI_TEST_REPORTS_DIR + '/videos'
        )

    exists = pod.check_file_exists('./test_passed')
    if not settings.KEEP_KAAS_UI_TEST_POD_AFTER:
        pod.delete()
    if not exists:
        pytest.fail("Some kaas UI tests failed. "
                    "See detailed report in artifacts")
